package com.diagbot.facade.data;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.data.ABehospitalInfoDTO;
import com.diagbot.entity.BehospitalInfo;
import com.diagbot.entity.MedicalRecord;
import com.diagbot.entity.QcType;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.facade.QcAbnormalFacade;
import com.diagbot.facade.QcTypeFacade;
import com.diagbot.service.impl.BehospitalInfoServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.util.TZDBConn;
import com.diagbot.vo.data.ABehospitalInfoVO;
import com.diagbot.vo.data.ADeleteFlagVO;
import com.diagbot.vo.data.APlaceFileVO;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;

@Component
public class ABehospitalInfoFacade extends BehospitalInfoServiceImpl {
    @Autowired
    @Qualifier("behospitalInfoServiceImpl")
    private BehospitalInfoServiceImpl behospitalInfoService;
    @Autowired
    private QcTypeFacade qcTypeFacade;
    @Autowired
    private AMedicalRecordFacade aMedicalRecordFacade;
    @Autowired
    private AMedAbnormalInfoFacade aMedAbnormalInfoFacade;
    @Autowired
    private QcAbnormalFacade qcAbnormalFacade;

    private TZDBConn tzDBConn = new TZDBConn();

    /**
     * 终末质控-同步前一天的入院记录
     */
    public void executeBehospital() {
        //String sql="select * from br_inpatientinfo where cjcxrq>=dateadd(day,-2,getdate()) and cjcxrq<=getdate()";
        String sql="select * from br_inpatientinfo where cjcxrq>=(select CONVERT(varchar,GETDATE()-2,23)) and cjcxrq<(select CONVERT(varchar,GETDATE(),23)) ORDER BY cjcxrq DESC";
        List<BehospitalInfo> behospitalInfoList = tzDBConn.getBehospitalInfo(sql);
        execute(behospitalInfoList);
    }

    /**
     * 同步前一天的入院记录
     */
    public void executeBehospitalPast() {
        String sql="select * from br_inpatientinfo where cjcxrq>=dateadd(day,-2,getdate()) and cjcxrq<=getdate()";
        List<BehospitalInfo> behospitalInfoList = tzDBConn.getBehospitalInfo(sql);
        execute(behospitalInfoList);
    }


    /**
     * 通过接口更新病人住院记录信息
     * @param list
     * @return
     */
    public RespDTO<List<ABehospitalInfoDTO>> executeBehospital(List<ABehospitalInfoVO> list) {
        if(list!=null && list.size()>0) {
            List<BehospitalInfo> behospitalInfoList = Lists.newArrayList();
            List<ABehospitalInfoDTO> behospitalInfoDtoList = Lists.newArrayList();
            list.stream().forEach(s->{
                BehospitalInfo behospitalInfo=new BehospitalInfo();
                BeanUtil.copyProperties(s,behospitalInfo);
                if(StringUtil.isNotBlank(s.getBirthday())) {
                    behospitalInfo.setBirthday(DateUtil.parseDate(s.getBirthday()));
                }
                if(StringUtil.isNotBlank(s.getLeaveHospitalDate())) {
                    behospitalInfo.setLeaveHospitalDate(DateUtil.parseDateTime(s.getLeaveHospitalDate()));
                }
                if(StringUtil.isNotBlank(s.getBehospitalDate())){
                    behospitalInfo.setBehospitalDate(DateUtil.parseDateTime(s.getBehospitalDate()));
                }
                if(StringUtil.isNotBlank(s.getPlacefileDate())){
                    behospitalInfo.setPlacefileDate(DateUtil.parseDateTime(s.getPlacefileDate()));
                }
                behospitalInfoList.add(behospitalInfo);

            });

            //循环验证数据有效性
            for (BehospitalInfo behospitalInfo:behospitalInfoList) {
                if("".equals(behospitalInfo.getBehospitalCode())) {
                    return RespDTO.onError("请输入病人住院编码!");
                }else if(behospitalInfo.getHospitalId()==null){
                    return RespDTO.onError("请输入医院编码!");
                }else if("".equals(behospitalInfo.getBehDeptId())){
                    return RespDTO.onError("请输入住院科室编码!");
                }else if("".equals(behospitalInfo.getBehDeptName())){
                    return RespDTO.onError("请输入住院科室名称!");
                }
            }
            execute(behospitalInfoList);
            behospitalInfoDtoList = BeanUtil.listCopyTo(list, ABehospitalInfoDTO.class);
            return RespDTO.onSuc(behospitalInfoDtoList);
        }else{
            return RespDTO.onError("未接收到数据!");
        }
    }

    public void execute(List<BehospitalInfo> behospitalInfoList){
        try {
            List<BehospitalInfo> addE = Lists.newLinkedList();
            List<BehospitalInfo> updateE = Lists.newLinkedList();
            behospitalInfoList.stream().forEach(s -> {
                BehospitalInfo behospitalInfo = this.getOne(new QueryWrapper<BehospitalInfo>()
                        .eq("behospital_code", s.getBehospitalCode())
                        .eq("hospital_id", s.getHospitalId())
                        .eq("is_deleted",IsDeleteEnum.N), false);
                if (behospitalInfo != null) {
                    s.setGmtModified(new Date());
                    s.setQcTypeId(initQcTypeId(s));
                    updateE.add(s);
                } else {
                    s.setGmtCreate(new Date());
                    s.setQcTypeId(initQcTypeId(s));
                    addE.add(s);
                }
            });
            if(addE.size()>0){
                behospitalInfoService.saveBatch(addE);
            }
            if(updateE.size()>0){
                behospitalInfoService.updateBatchByKey(updateE);
            }
        }catch (Exception e){
            log.error(e.getMessage(),e);
            aMedAbnormalInfoFacade.saveAbnormalInfo("病人住院登记", JSON.toJSONString(behospitalInfoList),"",e.getMessage());
        }
    }

    /**
     * 初始化QcTypeId
     * @param s
     * @return
     */
    private Long initQcTypeId(BehospitalInfo s){
        Long qcTypeId=Long.valueOf("0");
        MedicalRecord medicalRecord = aMedicalRecordFacade.getOne(new QueryWrapper<MedicalRecord>()
                .eq("behospital_code", s.getBehospitalCode())
                .eq("hospital_id", s.getHospitalId())
                .eq("mode_id", Long.valueOf("1"))
                .eq("is_deleted",IsDeleteEnum.N), false);
        if(medicalRecord!=null){
            QcType qcType=qcTypeFacade.getOne(new QueryWrapper<QcType>()
                    .eq("name", medicalRecord.getRecTitle())
                    .eq("hospital_id", medicalRecord.getHospitalId())
                    .eq("is_deleted",IsDeleteEnum.N));
            if(qcType!=null && qcType.getDefaultModule()==0){
                qcTypeId=qcType.getId();
            }else{
                QcType qcTypeStand=qcTypeFacade.getOne(new QueryWrapper<QcType>()
                        .eq("default_module", 1)
                        .eq("hospital_id", medicalRecord.getHospitalId())
                        .eq("is_deleted",IsDeleteEnum.N));
                if(qcTypeStand!=null){
                    qcTypeId=qcTypeStand.getId();
                }
            }
        }
        return qcTypeId;
    }

    public RespDTO deleteFlag(ADeleteFlagVO aDeleteFlagVO){
        try {
            //验证数据是否存在
            if(StringUtil.isEmpty(aDeleteFlagVO.getBehospitalCode())){
                return RespDTO.onError("请输入病人住院编码!");
            }else if(aDeleteFlagVO.getHospitalId()==null){
                return RespDTO.onError("请输入医院编码!");
            }else {
                UpdateWrapper<BehospitalInfo> updateWrapper=new UpdateWrapper<>();
                updateWrapper.eq("behospital_code", aDeleteFlagVO.getBehospitalCode())
                        .eq("hospital_id", aDeleteFlagVO.getHospitalId())
                        .eq("is_deleted",IsDeleteEnum.N)
                        .set("is_deleted",IsDeleteEnum.Y)
                        .set("gmt_modified", DateUtil.now());

                Boolean flag=update(new BehospitalInfo(),updateWrapper);
                //aMedAbnormalInfoFacade.saveAbnormalInfo("病人住院登记-删除",JSON.toJSONString(aDeleteFlagVO),"","删除成功!");
                return RespDTO.onSuc(flag);
            }
        }catch (Exception e){
            aMedAbnormalInfoFacade.saveAbnormalInfo("病人住院登记-删除",JSON.toJSONString(aDeleteFlagVO),"",e.getMessage());
            return RespDTO.onError(e.getMessage());
        }
    }

    public RespDTO placeFileFlag(APlaceFileVO aPlaceFileVO){
        try {
            //验证数据是否存在
            if(StringUtil.isBlank(aPlaceFileVO.getBehospitalCode())){
                return RespDTO.onError("请输入病人住院编码!");
            }else if(aPlaceFileVO.getHospitalId()==null){
                return RespDTO.onError("请输入医院编码!");
            }else{
                UpdateWrapper<BehospitalInfo> updateWrapper=new UpdateWrapper<>();
                updateWrapper.eq("behospital_code", aPlaceFileVO.getBehospitalCode())
                        .eq("hospital_id", aPlaceFileVO.getHospitalId())
                        .eq("is_deleted",IsDeleteEnum.N)
                        .set("is_placefile","1")
                        .set("placefile_date",aPlaceFileVO.getPlaceFileDate().isEmpty()?DateUtil.formatDateTime(DateUtil.now()):DateUtil.parseDateTime(aPlaceFileVO.getPlaceFileDate()))
                        .set("gmt_modified", DateUtil.now());

                Boolean flag=update(new BehospitalInfo(),updateWrapper);
                return RespDTO.onSuc(flag);
            }
        }catch (Exception e){
            aMedAbnormalInfoFacade.saveAbnormalInfo("病人住院登记-归档",JSON.toJSONString(aPlaceFileVO),"",e.getMessage());
            return RespDTO.onError(e.getMessage());
        }
    }
}
