package com.diagbot.facade.data;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.generator.config.rules.NamingStrategy;
import com.diagbot.dto.data.ColumnZhAndChDTO;
import com.diagbot.entity.MedBloodResult;
import com.diagbot.service.impl.MedBloodResultServiceImpl;
import com.diagbot.util.DateUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.MapUtil;
import com.diagbot.vo.data.AMedBloodResultVO;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.Map;

@Component
public class AMedBloodResultFacade extends MedBloodResultServiceImpl {
    @Autowired
    private ColumnFacade columnFacade;
    @Autowired
    private AMedAbnormalInfoFacade aMedAbnormalInfoFacade;

    public void getColumnZhAndCh(){
        List<ColumnZhAndChDTO> list=this.baseMapper.getColumnZhAndCh();
        list.forEach(s->{
            s.setEn(NamingStrategy.underlineToCamel(s.getEn()));
        });
        Map<String, String> columnMap= EntityUtil.makeMapWithKeyValue(list,"ch","en");
        Boolean flag=columnFacade.createColumn(columnMap,"medBloodResul");
        if(!flag){
            return;
        }
        //Map<String, String> map=columnFacade.getColumn("medBloodResul");
    }

    public void dealData(String columnMap,Map<String, Object> entityMap,Long hospitalId){
        try {
            AMedBloodResultVO medBloodResultVO=(AMedBloodResultVO) MapUtil.mapToObject(entityMap, AMedBloodResultVO.class);
            MedBloodResult medBloodResult=new MedBloodResult();
            BeanUtils.copyProperties(medBloodResultVO,medBloodResult);
            medBloodResult.setHospitalId(hospitalId);//医院编码
            medBloodResult.setAuditDate(DateUtil.parseDateTime(medBloodResultVO.getAuditDate()));//审核时间
            medBloodResult.setRecDate(DateUtil.parseDateTime(medBloodResultVO.getRecDate()));//记录时间
            medBloodResult.setRecordDate(DateUtil.parseDateTime(medBloodResultVO.getRecordDate()));//病历日期

            medBloodResult.setWholeData(columnMap);
            //判断数据库中是否存在，不存在insert
            if(isExist(medBloodResult)){
                medBloodResult.setGmtCreate(new Date());//记录创建时间
                this.save(medBloodResult);
            }else{
                medBloodResult.setGmtModified(new Date());//记录修改时间
                this.update(medBloodResult,new QueryWrapper<MedBloodResult>()
                        .eq("rec_id", medBloodResult.getRecId())
                        .eq("hospital_id", medBloodResult.getHospitalId())
                        .eq("behospital_code", medBloodResult.getBehospitalCode()));
            }

        }catch (Exception e){
            log.error(e.getMessage(),e);
            aMedAbnormalInfoFacade.saveAbnormalInfo("输血效果评价", JSON.toJSONString(entityMap),"",e.getMessage());
        }

    }

    public Boolean isExist(MedBloodResult medBloodResult){
        Boolean flag=true;
        int count = this.baseMapper.selectCount(new QueryWrapper<MedBloodResult>()
                .eq("rec_id", medBloodResult.getRecId())
                .eq("hospital_id", medBloodResult.getHospitalId())
                .eq("behospital_code", medBloodResult.getBehospitalCode()));
        if(count>0){
            flag=false;
        }
        return flag;
    }
}
