package com.diagbot.facade.data;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.generator.config.rules.NamingStrategy;
import com.diagbot.dto.data.ColumnZhAndChDTO;
import com.diagbot.entity.MedBloodTransfusion;
import com.diagbot.service.impl.MedBloodTransfusionServiceImpl;
import com.diagbot.util.DateUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.MapUtil;
import com.diagbot.vo.data.AMedBloodTransfusionVO;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.Map;

@Component
public class AMedBloodTransfusionFacade extends MedBloodTransfusionServiceImpl {
    @Autowired
    private ColumnFacade columnFacade;

    @Autowired
    private AMedAbnormalInfoFacade aMedAbnormalInfoFacade;

    public void getColumnZhAndCh(){
        List<ColumnZhAndChDTO> list=this.baseMapper.getColumnZhAndCh();
        list.forEach(s->{
            s.setEn(NamingStrategy.underlineToCamel(s.getEn()));
        });

        Map<String, String> columnMap= EntityUtil.makeMapWithKeyValue(list,"ch","en");
        Boolean flag=columnFacade.createColumn(columnMap,"medBloodTransfusion");

        //Map<String, String> map=columnFacade.getColumn("medConsultationNote");
        if(!flag){
            return;
        }
    }

    public void dealData(String columnMap,Map<String, Object> entityMap,Long hospitalId){
        try {
            AMedBloodTransfusionVO medBloodTransfusionVO=(AMedBloodTransfusionVO) MapUtil.mapToObject(entityMap, AMedBloodTransfusionVO.class);
            MedBloodTransfusion medBloodTransfusion=new MedBloodTransfusion();
            BeanUtils.copyProperties(medBloodTransfusionVO,medBloodTransfusion);
            medBloodTransfusion.setHospitalId(hospitalId);//医院编码
            medBloodTransfusion.setStartTime(DateUtil.parseDateTime(medBloodTransfusionVO.getStartTime()));
            medBloodTransfusion.setEndTime(DateUtil.parseDateTime(medBloodTransfusionVO.getEndTime()));
            medBloodTransfusion.setAuditDate(DateUtil.parseDateTime(medBloodTransfusionVO.getAuditDate()));//审核时间
            medBloodTransfusion.setRecDate(DateUtil.parseDateTime(medBloodTransfusionVO.getRecDate()));//记录时间
            medBloodTransfusion.setRecordDate(DateUtil.parseDateTime(medBloodTransfusionVO.getRecordDate()));//病历日期

            medBloodTransfusion.setWholeData(columnMap);

            //判断数据库中是否存在，不存在insert
            if(isExist(medBloodTransfusion)){
                medBloodTransfusion.setGmtCreate(new Date());//记录创建时间
                this.save(medBloodTransfusion);
            }else{
                medBloodTransfusion.setGmtModified(new Date());//记录修改时间
                this.update(medBloodTransfusion,new QueryWrapper<MedBloodTransfusion>()
                        .eq("rec_id", medBloodTransfusion.getRecId())
                        .eq("hospital_id", medBloodTransfusion.getHospitalId())
                        .eq("behospital_code", medBloodTransfusion.getBehospitalCode()));
            }

        }catch (Exception e){
            log.error(e.getMessage(),e);
            aMedAbnormalInfoFacade.saveAbnormalInfo("输血记录", JSON.toJSONString(entityMap),"",e.getMessage());
        }

    }

    public Boolean isExist(MedBloodTransfusion medBloodTransfusion){
        Boolean flag=true;
        int count = this.baseMapper.selectCount(new QueryWrapper<MedBloodTransfusion>()
                .eq("rec_id", medBloodTransfusion.getRecId())
                .eq("hospital_id", medBloodTransfusion.getHospitalId())
                .eq("behospital_code", medBloodTransfusion.getBehospitalCode()));
        if(count>0){
            flag=false;
        }
        return flag;
    }
}
