package com.diagbot.facade.data;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.generator.config.rules.NamingStrategy;
import com.diagbot.dto.data.ColumnZhAndChDTO;
import com.diagbot.entity.MedDeathDiscussion;
import com.diagbot.service.impl.MedDeathDiscussionServiceImpl;
import com.diagbot.util.DateUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.MapUtil;
import com.diagbot.vo.data.AMedDeathDiscussionVO;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.Map;

@Component
public class AMedDeathDiscussionFacade extends MedDeathDiscussionServiceImpl {
    @Autowired
    private ColumnFacade columnFacade;
    @Autowired
    private AMedAbnormalInfoFacade aMedAbnormalInfoFacade;

    public void getColumnZhAndCh(){
        List<ColumnZhAndChDTO> list=this.baseMapper.getColumnZhAndCh();
        list.forEach(s->{
            s.setEn(NamingStrategy.underlineToCamel(s.getEn()));
        });

        Map<String, String> columnMap= EntityUtil.makeMapWithKeyValue(list,"ch","en");
        Boolean flag=columnFacade.createColumn(columnMap,"medDeathDiscussion");

        //Map<String, String> map=columnFacade.getColumn("medConsultationNote");
        if(!flag){
            return;
        }
    }

    //死亡病例讨论记录
    public void dealData(String columnMap,Map<String, Object> entityMap,Long hospitalId){
        try {
            AMedDeathDiscussionVO medDeathDiscussionVO=(AMedDeathDiscussionVO) MapUtil.mapToObject(entityMap, AMedDeathDiscussionVO.class);
            MedDeathDiscussion medDeathDiscussion=new MedDeathDiscussion();
            BeanUtils.copyProperties(medDeathDiscussionVO,medDeathDiscussion);
            medDeathDiscussion.setHospitalId(hospitalId);//医院编码

            medDeathDiscussion.setAuditDate(DateUtil.parseDateTime(medDeathDiscussionVO.getAuditDate()));//审核时间
            medDeathDiscussion.setRecDate(DateUtil.parseDateTime(medDeathDiscussionVO.getRecDate()));//记录时间
            medDeathDiscussion.setDeathDate(DateUtil.parseDateTime(medDeathDiscussionVO.getDeathDate()));//死亡时间
            medDeathDiscussion.setDiscussDate(DateUtil.parseDate(medDeathDiscussionVO.getDiscussDate()));//讨论时间
            medDeathDiscussion.setRecordDate(DateUtil.parseDateTime(medDeathDiscussionVO.getRecordDate()));//病历日期

            medDeathDiscussion.setWholeData(columnMap);
            //判断数据库中是否存在，不存在insert
            if(isExist(medDeathDiscussion)){
                medDeathDiscussion.setGmtCreate(new Date());//记录创建时间
                this.save(medDeathDiscussion);
            }else{
                medDeathDiscussion.setGmtModified(new Date());//记录修改时间
                this.update(medDeathDiscussion,new QueryWrapper<MedDeathDiscussion>()
                        .eq("rec_id", medDeathDiscussion.getRecId())
                        .eq("hospital_id", medDeathDiscussion.getHospitalId())
                        .eq("behospital_code", medDeathDiscussion.getBehospitalCode()));
            }

        }catch (Exception e){
            log.error(e.getMessage(),e);
            aMedAbnormalInfoFacade.saveAbnormalInfo("死亡病例讨论记录", JSON.toJSONString(entityMap),"",e.getMessage());
        }

    }

    public Boolean isExist(MedDeathDiscussion medDeathDiscussion){
        Boolean flag=true;
        int count = this.baseMapper.selectCount(new QueryWrapper<MedDeathDiscussion>()
                .eq("rec_id", medDeathDiscussion.getRecId())
                .eq("hospital_id", medDeathDiscussion.getHospitalId())
                .eq("behospital_code", medDeathDiscussion.getBehospitalCode()));
        if(count>0){
            flag=false;
        }
        return flag;
    }
}
