package com.diagbot.facade.data;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.generator.config.rules.NamingStrategy;
import com.diagbot.dto.data.ColumnZhAndChDTO;
import com.diagbot.entity.MedRescueNote;
import com.diagbot.service.impl.MedRescueNoteServiceImpl;
import com.diagbot.util.DateUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.MapUtil;
import com.diagbot.vo.data.AMedRescueNoteVO;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.Map;

@Component
public class AMedRescueNoteFacade extends MedRescueNoteServiceImpl {
    @Autowired
    private ColumnFacade columnFacade;
    @Autowired
    private AMedAbnormalInfoFacade aMedAbnormalInfoFacade;

    public void getColumnZhAndCh(){
        List<ColumnZhAndChDTO> list=this.baseMapper.getColumnZhAndCh();
        list.forEach(s->{
            s.setEn(NamingStrategy.underlineToCamel(s.getEn()));
        });

        Map<String, String> columnMap= EntityUtil.makeMapWithKeyValue(list,"ch","en");
        Boolean flag=columnFacade.createColumn(columnMap,"medRescueNote");

        //Map<String, String> map=columnFacade.getColumn("medConsultationNote");
        if(!flag){
            return;
        }
    }

    public void dealData(String columnMap,Map<String, Object> entityMap,Long hospitalId){
        try {
            AMedRescueNoteVO medRescueNoteVO=(AMedRescueNoteVO) MapUtil.mapToObject(entityMap, AMedRescueNoteVO.class);
            MedRescueNote medRescueNote=new MedRescueNote();
            BeanUtils.copyProperties(medRescueNoteVO,medRescueNote);
            medRescueNote.setHospitalId(hospitalId);//医院编码

            medRescueNote.setAuditDate(DateUtil.parseDateTime(medRescueNoteVO.getAuditDate()));//审核时间
            medRescueNote.setRecDate(DateUtil.parseDateTime(medRescueNoteVO.getRecDate()));//记录时间
            medRescueNote.setStartTime(DateUtil.parseDateTime(medRescueNoteVO.getStartTime()));//开始时间
            medRescueNote.setEndTime(DateUtil.parseDate(medRescueNoteVO.getEndTime()));//结束时间
            medRescueNote.setRecordDate(DateUtil.parseDateTime(medRescueNoteVO.getRecordDate()));//病历日期

            medRescueNote.setWholeData(columnMap);
            //判断数据库中是否存在，不存在insert
            if(isExist(medRescueNote)){
                medRescueNote.setGmtCreate(new Date());//记录创建时间
                this.save(medRescueNote);
            }else{
                medRescueNote.setGmtModified(new Date());//记录修改时间
                this.update(medRescueNote,new QueryWrapper<MedRescueNote>()
                        .eq("rec_id", medRescueNote.getRecId())
                        .eq("hospital_id", medRescueNote.getHospitalId())
                        .eq("behospital_code", medRescueNote.getBehospitalCode()));
            }

        }catch (Exception e){
            log.error(e.getMessage(),e);
            aMedAbnormalInfoFacade.saveAbnormalInfo("抢救记录", JSON.toJSONString(entityMap),"",e.getMessage());
        }

    }

    public Boolean isExist(MedRescueNote medRescueNote){
        Boolean flag=true;
        int count = this.baseMapper.selectCount(new QueryWrapper<MedRescueNote>()
                .eq("rec_id", medRescueNote.getRecId())
                .eq("hospital_id", medRescueNote.getHospitalId())
                .eq("behospital_code", medRescueNote.getBehospitalCode()));
        if(count>0){
            flag=false;
        }
        return flag;
    }
}
