package com.diagbot.facade.data;

import com.alibaba.fastjson.JSON;
import com.diagbot.dto.OutputInfo;
import com.diagbot.dto.Response;
import com.diagbot.enums.CacheKeyEnum;
import com.diagbot.facade.QcClientFacade;
import com.diagbot.service.impl.ColumnServiceImpl;
import com.diagbot.vo.QueryVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@Component
public class ColumnFacade extends ColumnServiceImpl {
    @Autowired
    private QcClientFacade qcClientFacade;
    @Autowired
    private AMedAbnormalInfoFacade aMedAbnormalInfoFacade;

    /**
     * 解析
     * @param queryVo
     */
    public void analyseRec(QueryVo queryVo){
        try {
            Response<OutputInfo> resp=qcClientFacade.analyseField(queryVo);
            if(resp.getData()!=null && resp.getData().getPageData()!=null){
                Map<String, Object> pageData=resp.getData().getPageData();
                pageData.forEach((key,value)->{
                    List<Map<String, Object>> listMap=(List<Map<String,Object>>)resp.getData().getPageData().get(key);
                    Map<String, String> columnMap=this.getColumn(CacheKeyEnum.getName(key));

                    Map<String, Object> entityMap=new HashMap<String, Object>();
                    listMap.forEach(s->{
                        columnMap.forEach((k, v) -> {
                            entityMap.put(v,s.get(k));
                        });
                        //数据解析
                        analysisData(key,entityMap,Long.valueOf(queryVo.getHospitalId()),JSON.toJSONString(s));
                    });
                });
            }
        }catch (Exception e){
            log.error(e.getMessage(),e);
            aMedAbnormalInfoFacade.saveAbnormalInfo("数据解析", JSON.toJSONString(queryVo),"",e.getMessage());
        }

    }

    /**
     * 数据解析
     * @param modelName
     * @param entityMap
     * @param hospitalId
     */
    private void analysisData(String modelName,Map<String, Object> entityMap,Long hospitalId,String columnMap){
        try {
            if("入院记录".equals(modelName)){
                aMedAdmissionNoteFacade.dealData(columnMap,entityMap,Long.valueOf(hospitalId));
            }else if("首次病程录".equals(modelName)){
                aMedFirstRecordFacade.dealData(columnMap,entityMap,Long.valueOf(hospitalId));
            }else if("输血后效果评价".equals(modelName)){
                aMedBloodResultFacade.dealData(columnMap,entityMap,Long.valueOf(hospitalId));
            }else if("输血/血制品病程记录".equals(modelName)){
                aMedBloodTransfusionFacade.dealData(columnMap,entityMap,Long.valueOf(hospitalId));
            }else if("转入记录".equals(modelName)){
                aMedTransferInNoteFacade.dealData(columnMap,entityMap,Long.valueOf(hospitalId));
            }else if("转出记录".equals(modelName)){
                aMedTransferOutNoteFacade.dealData(columnMap,entityMap,Long.valueOf(hospitalId));
            }else if("病危通知书".equals(modelName)){
                aMedIllCriticallyFacade.dealData(columnMap,entityMap,Long.valueOf(hospitalId));
            }else if("病重通知书".equals(modelName)){
                aMedIllSeriouslFacade.dealData(columnMap,entityMap,Long.valueOf(hospitalId));
            }else if("疑难病例讨论记录".equals(modelName)){
                aMedDifficultCaseFacade.dealData(columnMap,entityMap,Long.valueOf(hospitalId));
            }else if("死亡记录".equals(modelName)){
                aMedDeathNoteFacade.dealData(columnMap,entityMap,Long.valueOf(hospitalId));
            }else if("死亡病例讨论记录".equals(modelName)){
                aMedDeathDiscussionFacade.dealData(columnMap,entityMap,Long.valueOf(hospitalId));
            }else if("查房记录".equals(modelName)){
                aMedWardRecordFacade.dealData(columnMap,entityMap,Long.valueOf(hospitalId));
            }else if("术后首次病程及谈话记录".equals(modelName)){
                aMedOperativeFirstRecord.dealData(columnMap,entityMap,Long.valueOf(hospitalId));
            }else if("术前讨论、术前小结".equals(modelName)){
                aMedPreoperativeDiscussionFacade.dealData(columnMap,entityMap,Long.valueOf(hospitalId));
            }else if("手术记录".equals(modelName)){
                aMedOperativeNoteFacade.dealData(columnMap,entityMap,Long.valueOf(hospitalId));
            }else if("抢救记录".equals(modelName)){
                aMedRescueNoteFacade.dealData(columnMap,entityMap,Long.valueOf(hospitalId));
            }else if("危急值记录".equals(modelName)){
                aMedCrisisNoteFacade.dealData(columnMap,entityMap,Long.valueOf(hospitalId));
            }else if("出院小结".equals(modelName)){
                aMedLeaveHospitalFacade.dealData(columnMap,entityMap,Long.valueOf(hospitalId));
            }else if("会诊申请单".equals(modelName)){
                aMedConsultationApplyFacade.dealData(columnMap,entityMap,Long.valueOf(hospitalId));
            }else if("会诊结果单".equals(modelName)){
                aMedConsultationResultFacade.dealData(columnMap,entityMap,Long.valueOf(hospitalId));
            }else if("会诊单".equals(modelName)){
                aMedConsultationNoteFacade.dealData(columnMap,entityMap,Long.valueOf(hospitalId));
            }else if("阶段小结".equals(modelName)){
                aMedPeriodConclusionFacade.dealData(columnMap,entityMap,Long.valueOf(hospitalId));
            }
            log.info(modelName+"---- 解析完成！");
        }catch (Exception e){
            log.error(e.getMessage(),e);
            aMedAbnormalInfoFacade.saveAbnormalInfo("数据解析异常", JSON.toJSONString(columnMap),"",e.getMessage());
        }
    }

    @Autowired
    private AMedAdmissionNoteFacade aMedAdmissionNoteFacade;
    @Autowired
    private AMedFirstRecordFacade aMedFirstRecordFacade;
    @Autowired
    private AMedPeriodConclusionFacade aMedPeriodConclusionFacade;
    @Autowired
    private AMedBloodResultFacade aMedBloodResultFacade;
    @Autowired
    private AMedBloodTransfusionFacade aMedBloodTransfusionFacade;
    @Autowired
    private AMedTransferInNoteFacade aMedTransferInNoteFacade;
    @Autowired
    private AMedTransferOutNoteFacade aMedTransferOutNoteFacade;
    @Autowired
    private AMedIllSeriouslFacade aMedIllSeriouslFacade;
    @Autowired
    private AMedIllCriticallyFacade aMedIllCriticallyFacade;
    @Autowired
    private AMedDifficultCaseFacade aMedDifficultCaseFacade;
    @Autowired
    private AMedDeathNoteFacade aMedDeathNoteFacade;
    @Autowired
    private AMedDeathDiscussionFacade aMedDeathDiscussionFacade;
    @Autowired
    private AMedWardRecordFacade aMedWardRecordFacade;
    @Autowired
    private AMedOperativeFirstRecord aMedOperativeFirstRecord;
    @Autowired
    private AMedPreoperativeDiscussionFacade aMedPreoperativeDiscussionFacade;
    @Autowired
    private AMedOperativeNoteFacade aMedOperativeNoteFacade;
    @Autowired
    private AMedRescueNoteFacade aMedRescueNoteFacade;
    @Autowired
    private AMedCrisisNoteFacade aMedCrisisNoteFacade;
    @Autowired
    private AMedLeaveHospitalFacade aMedLeaveHospitalFacade;
    @Autowired
    private AMedConsultationApplyFacade aMedConsultationApplyFacade;
    @Autowired
    private AMedConsultationResultFacade aMedConsultationResultFacade;
    @Autowired
    private AMedConsultationNoteFacade aMedConsultationNoteFacade;
}
